<?php
declare(strict_types = 1);

/**
 * Modèle de description pour catégories et fichiers.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Description
{
	/**
	 * Données de la catégorie ou du fichier.
	 *
	 * @var array
	 */
	private static $_data;

	/**
	 * Métadonnées du fichier.
	 *
	 * @var object
	 */
	private static $_metadata;



	/**
	 * Applique le modèle de description pour catégorie ou fichier.
	 *
	 * @param string $type
	 * @param array $data
	 *
	 * @return string
	 */
	public static function model(string $type, array &$data): string
	{
		self::$_data = &$data;
		self::$_metadata = NULL;

		$model = $type == 'cat'
			? Config::$params['categories_description_model_text']
			: Config::$params['items_description_model_text'];
		do
		{
			$test = $model;
			$model = preg_replace_callback(
				'`{IF\(([A-Z]{2,20})\)}[^$]*?{ENDIF\(\1\)}`i',
				'Description::_' . $type . 'If', $model);
		}
		while ($test != $model);
		$model = preg_replace_callback('`{([A-Z]{2,20})}`i',
			'Description::_' . $type . 'Var', $model);

		if ($type == 'item')
		{
			do
			{
				$test = $model;
				$model = preg_replace_callback(
					'`{IF\(((EXIF|IPTC|XMP):([A-Z0-9]{2,20}))\)}[^$]*?{ENDIF\(\1\)}`i',
					'Description::_itemIfMeta', $model);
			}
			while ($test != $model);
			$model = preg_replace_callback('`{(EXIF|IPTC|XMP):([A-Z0-9]{2,20})}`i',
				'Description::_itemVarMeta', $model);
		}

		return $model;
	}



	/**
	 * Conditions pour les informations de catégories.
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _catIf(array $m)
	{
		$replace = FALSE;

		switch ($m[1])
		{
			case 'ID' :
			case 'URL' :
				$replace = TRUE;
				break;

			case 'DESCRIPTION' :
				$replace = !Utility::isEmpty((string) self::$_data['cat_desc']);
				break;

			case 'TITLE' :
				$replace = !Utility::isEmpty((string) self::$_data['cat_name']);
				break;
		}

		if ($replace)
		{
			return substr($m[0], 6 + strlen($m[1]), -9 - strlen($m[1]));
		}
	}

	/**
	 * Informations de catégories.
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _catVar(array $m)
	{
		switch ($m[1])
		{
			case 'DESCRIPTION' :
				return self::$_data['cat_desc'];

			case 'ID' :
				return self::$_data['cat_id'];

			case 'TITLE' :
				return self::$_data['cat_name'];

			case 'URL' :
				return GALLERY_HOST . App::getURL(self::$_data['cat_type'] . '/'
					. self::$_data['cat_id'] . '-' . self::$_data['cat_url']);
		}

		return $m[0];
	}

	/**
	 * Conditions pour les informations de fichiers.
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _itemIf(array $m)
	{
		$replace = FALSE;

		switch ($m[1])
		{
			case 'DATEADDED' :
			case 'DATETIMEADDED' :
			case 'FILENAME' :
			case 'FILETYPE' :
			case 'ID' :
			case 'URL' :
			case 'USERNAME' :
				$replace = TRUE;
				break;

			case 'DATECREATED' :
			case 'DATETIMECREATED' :
				$replace = self::$_data['item_crtdt'];
				break;

			case 'DESCRIPTION' :
				$replace = !Utility::isEmpty((string) self::$_data['item_desc']);
				break;

			case 'DURATION' :
				$replace = (int) self::$_data['item_duration'];
				break;

			case 'FILESIZE' :
				$replace = (int) self::$_data['item_filesize'];
				break;

			case 'HEIGHT' :
				$replace = (int) self::$_data['item_height'];
				break;

			case 'TITLE' :
				$replace = !Utility::isEmpty((string) self::$_data['item_name']);
				break;

			case 'WIDTH' :
				$replace = (int) self::$_data['item_width'];
				break;
		}

		if ($replace)
		{
			return substr($m[0], 6 + strlen($m[1]), -9 - strlen($m[1]));
		}
	}

	/**
	 * Conditions pour les informations de fichiers (métadonnées).
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _itemIfMeta(array $m)
	{
		if (self::_itemVarMeta([$m[0], $m[2], $m[3]]))
		{
			return substr($m[0], 6 + strlen($m[1]), -9 - strlen($m[1]));
		}
	}

	/**
	 * Informations de fichiers.
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _itemVar(array $m)
	{
		switch ($m[1])
		{
			case 'DATEADDED' :
			case 'DATEPUBLISHED' :
				return L10N::dt(__('%A %d %B %Y'), self::$_data['item_pubdt']);

			case 'DATETIMEADDED' :
			case 'DATETIMEPUBLISHED' :
				return L10N::dt(__('%A %d %B %Y à %H:%M:%S'), self::$_data['item_pubdt']);

			case 'DATECREATED' :
				return self::$_data['item_crtdt']
					? L10N::dt(__('%A %d %B %Y'), self::$_data['item_crtdt'], FALSE)
					: '';

			case 'DATETIMECREATED' :
				return self::$_data['item_crtdt']
					? L10N::dt(__('%A %d %B %Y à %H:%M:%S'), self::$_data['item_crtdt'], FALSE)
					: '';

			case 'DESCRIPTION' :
				return self::$_data['item_desc'];

			case 'DURATION' :
				return self::$_data['item_duration']
					? App::formatDuration(self::$_data['item_duration'])
					: '00:00';

			case 'FILENAME' :
				return basename(self::$_data['item_path']);

			case 'FILESIZE' :
				return self::$_data['item_filesize']
					? L10N::formatFilesize(self::$_data['item_filesize'])
					: 0;

			case 'FILETYPE' :
				return Item::getTypeText(self::$_data['item_type']);

			case 'ID' :
				return self::$_data['item_id'];

			case 'HEIGHT' :
				return self::$_data['item_height'];

			case 'TITLE' :
				return self::$_data['item_name'];

			case 'URL' :
				return GALLERY_HOST . App::getURL('item/'
					. self::$_data['item_id'] . '-' . self::$_data['item_url']);

			case 'USERNAME' :
				return User::getNickname(
					self::$_data['user_login'], self::$_data['user_nickname']
				);

			case 'WIDTH' :
				return self::$_data['item_width'];
		}

		return $m[0];
	}

	/**
	 * Informations de fichiers (métadonnées).
	 *
	 * @param array $m
	 *
	 * @return mixed
	 */
	private static function _itemVarMeta(array $m)
	{
		if (!self::$_metadata)
		{
			self::$_metadata = new Metadata(
				CONF_ALBUMS_PATH . '/' . self::$_data['item_path']
			);
		}

		if ($m[1] == 'EXIF')
		{
			$formated = self::$_metadata->getExifFormatedInfo($m[2],
				Config::$params['exif_params']);
		}

		if ($m[1] == 'IPTC')
		{
			$formated = self::$_metadata->getIptcFormatedInfo($m[2],
				Config::$params['iptc_params']);
		}

		if ($m[1] == 'XMP')
		{
			$formated = self::$_metadata->getXmpFormatedInfo($m[2],
				Config::$params['xmp_params']);
		}

		return $formated['value'] ?? NULL;
	}
}
?>